﻿using DataModel.Database;
using DataModel.Database.Common;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;

namespace Processor;

public static class TextElementProcessor
{
    public static async Task<List<TextElement>> GetTextElements(Section section)
    {
        await using var db = ProcessorMeta.ServiceProvider.GetRequiredService<DataContext>();
        return await db.TextElements.Where(n => n.Section == section).OrderBy(n => n.Name).ThenBy(n => n.Id).ToListAsync();
    }

    public static async Task<ProcessorResult<TextElement>> AddTextElement(string? currentSectionKey, string elementName)
    {
        await using var db = ProcessorMeta.ServiceProvider.GetRequiredService<DataContext>();
        
        if(string.IsNullOrWhiteSpace(currentSectionKey))
            throw new ArgumentOutOfRangeException(nameof(currentSectionKey));
        
        var currentSection = await db.Sections.FirstOrDefaultAsync(n => n.DataKey == currentSectionKey);
        if (currentSection is null)
            throw new ArgumentOutOfRangeException(nameof(currentSectionKey));
        
        // Generate a code:
        var code = await Utils.GenerateCode(elementName, db.TextElements, (n, code) => n.Section == currentSection && n.Code == code);
        
        var textElement = new TextElement
        {
            Name = elementName,
            Code = code,
            Section = currentSection,
        };
        
        // Add the new element to the database:
        await db.TextElements.AddAsync(textElement);

        try
        {
            // Save the changes:
            await db.SaveChangesAsync();
            return new ProcessorResult<TextElement>(textElement);
        }
        catch (DbUpdateException updateException)
        {
            return updateException.ToProcessorResult<TextElement>();
        }
    }
}